@@ -15,7 +15,7 @@ from typing import Dict, Iterator, List, NewType, Optional, Set, Tuple, TypeVar,
 
 from breezy import errors as bzr_errors
 from breezy import repository, tsort
-from breezy.builtins import cmd_branch
+from breezy.builtins import cmd_branch, cmd_upgrade
 from breezy.bzr import bzrdir
 from breezy.bzr.branch import Branch as BzrBranch
 from breezy.bzr.inventory import Inventory, InventoryEntry
@@ -78,11 +78,6 @@ older_repository_formats = {
 expected_repository_format = b"Bazaar repository format 2a (needs bzr 1.16 or later)\n"
 
 
-class RepositoryNeedsUpgrade(Exception):
-    """The repository we're trying to load is using an old format.
-    We only support format 2a (the most recent), see `brz help upgrade`"""
-
-
 class UnknownRepositoryFormat(Exception):
     """The repository we're trying to load is using an unknown format.
     It's possible (though unlikely) that a new format has come out, we should
@@ -269,6 +264,17 @@ class BazaarLoader(BaseLoader):
         if self.repo is not None:
             self.repo.unlock()
 
+    def get_repo_and_branch(self) -> Tuple[repository.Repository, BzrBranch]:
+        _, branch, repo, _ = bzrdir.BzrDir.open_containing_tree_branch_or_repository(
+            self._repo_directory
+        )
+        return repo, branch
+
+    def run_upgrade(self):
+        """Upgrade both repository and branch to the most recent supported version
+        to be compatible with the loader."""
+        cmd_upgrade().run(self._repo_directory, clean=True)
+
     def fetch_data(self) -> bool:
         """Fetch the data from the source the loader is currently loading
 
@@ -303,17 +309,31 @@ class BazaarLoader(BaseLoader):
             self.log.debug("Using local directory '%s'", self.directory)
             self._repo_directory = self.directory
 
-        res = bzrdir.BzrDir.open_containing_tree_branch_or_repository(
-            self._repo_directory
-        )
-        (_tree, _branch, repo, _relpath) = res
+        repo, branch = self.get_repo_and_branch()
         repository_format = repo._format.as_string()  # lies about being a string
+
         if not repository_format == expected_repository_format:
             if repository_format in older_repository_formats:
-                raise RepositoryNeedsUpgrade()
+                self.log.debug(
+                    "Upgrading repository from format '%s'",
+                    repository_format.decode("ascii").strip("\n"),
+                )
+                self.run_upgrade()
+                repo, branch = self.get_repo_and_branch()
             else:
                 raise UnknownRepositoryFormat()
 
+        if not branch.supports_tags():
+            # Some repos have the right format marker but their branches do not
+            # support tags
+            self.log.debug("Branch does not support tags, upgrading")
+            self.run_upgrade()
+            repo, branch = self.get_repo_and_branch()
+            # We could set the branch here directly, but we want to run the
+            # sanity checks in the `self.branch` property, so let's make sure
+            # we invalidate the "cache".
+            self._branch = None
+
         self.repo = repo
         self.repo.lock_read()
         self.head_revision_id  # set the property
@@ -678,7 +698,7 @@ class BazaarLoader(BaseLoader):
     @property
     def tags(self) -> Optional[Dict[bytes, BzrRevisionId]]:
         assert self.repo is not None
-        if self._tags is None and self.branch.supports_tags():
+        if self._tags is None:
             self._tags = {
                 n.encode(): r for n, r in self.branch.tags.get_tag_dict().items()
             }
